// QWeb - An SGML Web Browser
// Copyright (C) 1997  Sean Vyain
// svyain@mail.tds.net
// smvyain@softart.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#ifndef _FormObjectRenderer_h_
#define _FormObjectRenderer_h_

#include "Renderer.h"

class FormRenderer;

//: The FormObjectRenderer is the abstract base class for all input form widgets.
class FormObjectRenderer : public Renderer {
    Q_OBJECT
public:
    //. Register this FormObject with the nearest enclosing FormRenderer.  When
    //. the form is submitted, the FormRenderer will ask this FormObject for
    //. its value.
    FormObjectRenderer( Canvas*     canvas,
                        int         clipWidth=0,
                        QObject*    parent=0,
                        const char* name=0 );

    virtual ~FormObjectRenderer();

    static int count;

    //. Return the FormObject's value in the form "name=value".  All special
    //. characters in the string are escaped before returning.  This function
    //. may return NULL if the FormObject does not contribute a value to the
    //. form.
    virtual QString value() = 0;

    virtual void reset() = 0;

    //. Find the nearest enclosing FormRenderer by walking up the list of
    //. parent objects.
    FormRenderer* form();
};

#endif
